<?php
namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Log;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\View\View;

class ProfileController extends Controller
{
    /**
     * Display the user's profile view.
     */
    public function view(Request $request): View
    {
        return view('profile.view', [
            'user' => $request->user(),
        ]);
    }

    /**
     * Display the user's profile edit form.
     */
    public function edit(Request $request): View
    {
        return view('profile.edit', [
            'user' => $request->user(),
            'defaultAvatars' => $this->getDefaultAvatars(),
        ]);
    }

    /**
     * Update the user's profile information.
     */
    public function update($locale, Request $request): RedirectResponse
    {
        $request->validate([
            'avatar' => 'nullable|image|mimes:jpg,jpeg,png,gif,svg|max:2048',
        ]);

        $user = $request->user();
        $user->fill($request->except('avatar'));

        if ($request->has('avatar_default') && !empty($request->avatar_default)) {
            $this->setDefaultAvatar($request, $user);
        } elseif ($request->hasFile('avatar')) {
            $this->updateAvatar($request, $user);
        }

        if ($user->isDirty('email')) {
            $user->email_verified_at = null;
        }

        $user->save();

        return Redirect::route('profile.edit', $locale)->with('status', 'profile-updated');
    }

    /**
     * Set the user's avatar to a default avatar.
     */
    protected function setDefaultAvatar(Request $request, $user)
    {
        if ($user->avatar && !str_starts_with($user->avatar, 'avatars/default/')) {
            Storage::disk('public')->delete($user->avatar);
            Log::info('Old custom avatar deleted');
        }
        $user->avatar = 'avatars/default/' . $request->avatar_default;
    }

    /**
     * Update the user's avatar.
     */
    protected function updateAvatar(Request $request, $user)
    {
        Log::info('Avatar update initiated'); 

        // Delete old custom avatar if exists
        if ($user->avatar && !str_starts_with($user->avatar, 'avatars/default/')) {
            Storage::disk('public')->delete($user->avatar);
            Log::info('Old custom avatar deleted');
        }

        // Store new avatar
        $avatarPath = $request->file('avatar')->store('avatars', 'public');
        Log::info('Avatar stored at: ' . $avatarPath); 

        $user->avatar = $avatarPath;
    }

    /**
     * Get the list of default avatars.
     */
    protected function getDefaultAvatars()
    {
        return [
            asset('assets/images/profile/user-1.jpg'),
            asset('assets/images/profile/user-2.jpg'),
            asset('assets/images/profile/user-3.jpg'),
            asset('assets/images/profile/user-4.jpg'),
            asset('assets/images/profile/user-5.jpg'),
            asset('assets/images/profile/user-6.jpg'),
            asset('assets/images/profile/user-7.jpg'),
            asset('assets/images/profile/user-8.jpg'),
            asset('assets/images/profile/user-9.jpg'),
            asset('assets/images/profile/user-10.jpg'),
            asset('assets/images/profile/user-11.jpg'),
            asset('assets/images/profile/user-12.jpg'),
        ];
    }

    /**
     * Delete the user's account.
     */
    public function destroy(Request $request): RedirectResponse
    {
        $request->validateWithBag('userDeletion', [
            'password' => ['required', 'current_password'],
        ]);

        $user = $request->user();

        Auth::logout();

        $user->delete();

        $request->session()->invalidate();
        $request->session()->regenerateToken();

        return Redirect::to('/');
    }

    /**
     * Change password.
     */
    public function change_password(Request $request): View
    {
        return view('profile.update-password', [
            'user' => $request->user(),
        ]);
    }
}
