<?php

namespace App\Http\Controllers\LibraryVideo;
use App\Http\Controllers\Controller;
use App\Models\LibraryVideo;
use App\Models\LibraryVideoHistory;
use App\Models\Category;
use App\Models\UserTraining;
use App\Models\Department;
use App\Models\Work;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\Request;
use \Carbon\Carbon;

class VideoLibraryController extends Controller
{
    /**
     * Affiche la liste des vidéos de formation dans la bibliothèque.
     */
    public function index(Request $request, $locale, $category_id)
    {

        $category = Category::findOrFail($category_id);
        $departments = Department::where('company_id', $this->company->id)->get();
        $jobs = Work::all();

        // Récupère les formations utilisateur associées à la catégorie
        $userTrainings = UserTraining::where('user_id', $this->user->id)
            ->where('company_id', $this->company->id)
            ->where('category_id', $category_id)
            ->get()
            ->map(function ($userTraining) {
                $userTraining->training->image = $this->unserialize($userTraining->training->image);
                $trackRecordDuration = $userTraining->track_records->sum('duration');
                $userTraining->track_record_duration_second = $trackRecordDuration;
                $userTraining->track_record_duration = $this->sec_to_time_full($trackRecordDuration);
                $completionInfo = $this->completed_info($userTraining);
                $userTraining->is_completed = $completionInfo['completed'];
                $userTraining->date = $completionInfo['date'];
                return $userTraining;
            });
        // Construction de la requête pour récupérer les vidéos filtrées
        // Début de la requête pour les vidéos
        $query = LibraryVideo::query();

        // Filtre par nom de vidéo
        if ($request->filled('videoName')) {
            // Recherche par nom de vidéo avec un LIKE pour la correspondance partielle
            $query->where('name', 'like', '%' . $request->videoName . '%');
        }

        // Filtre par département
        if ($request->filled('department')) {
            // Vérifie si le département existe et applique le filtre
            $query->where('department_id', $request->department);
        }

        // Filtre par job (poste)
        if ($request->filled('job')) {
            // Vérifie si le job est sélectionné et applique le filtre par 'work_id'
            $query->where('work_id', $request->job);
        }

        // Filtre par date de création
        if ($request->filled('creationDate')) {
            // Assurez-vous que la date est au format correct, puis appliquez le filtre
            $query->whereDate('date_creation', $request->creationDate);
        }
        //dd($request->all());
        // Récupère les vidéos filtrées
        $libraryVideos = $query->get()->map(function ($video) {
            // Vérifie si le champ filename est vide et utilise l'URL si c'est le cas
            $video->filename = empty($video->filename) ? $video->url : Storage::url($video->filename);

            // Vérifie si l'URL est un lien YouTube et transforme-le en lien d'intégration
            $video->url = $this->getYouTubeEmbedUrl($video->url);

            return $video;
        });
       

        return view('libraryvideo.index', [
            'tutorial_video' => $this->get_tutorial_video($category, $locale),
            'category' => $category,
            'user_trainings' => $userTrainings,
            'departments' => $departments,
            'category_id' => $category_id,
            'jobs' => $jobs,
            'libraryVideo' => $libraryVideos,
        ]);
    }

    /**
     * Affiche le formulaire pour créer une nouvelle vidéo.
     */
    public function create($locale, $category_id)
    {
        $departments = Department::where('company_id', $this->company->id)->get();
        $jobs = Work::all();
        $category = Category::findOrFail($category_id);

        return view('libraryvideo.create', compact('departments', 'jobs', 'category'));
    }
    public function list($locale, $category_id)
    {
        $libraryVideos = LibraryVideo::all();
        $video = $libraryVideos->pluck('id');
        $departments = Department::where('company_id', $this->company->id)->get();
        $works = Work::all();
        //$work = Work::where('company_id', $this->company->id)->get();
        $category = Category::findOrFail($category_id);


        return view('libraryvideo.liste', compact('departments', 'works', 'category', 'libraryVideos', 'video'));
    }

    /**
     * Stocke une nouvelle vidéo dans la bibliothèque.
     */
    public function store(Request $request, $locale)
    {
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'department_id' => 'nullable|integer',
            'work_id' => 'nullable|integer',
            'date_creation' => 'nullable|date',
            'image' => 'nullable|image|max:20480',
            'video' => 'nullable|mimes:mp4,mov,ogg,qt|max:50000',
            'url' => 'nullable|string|max:512',
            'display_order' => 'required|integer',
        ]);

        $validatedData['company_id'] = $this->company->id;

        if ($request->hasFile('image')) {
            $validatedData['image'] = $request->file('image')->store('images', 'public');
        }

        if ($request->hasFile('video')) {
            $validatedData['filename'] = $request->file('video')->store('videos', 'public');
        }

        try {
            LibraryVideo::create($validatedData);
            return redirect()->route('training-library_video.index', ['locale' => $locale])
                ->with('success', __('messages.video_added_successfully'));
        } catch (\Exception $e) {
            \Log::error('Erreur lors de l\'insertion de la vidéo : ' . $e->getMessage());
            return redirect()->back()->withErrors(__('messages.error_adding_video'))->withInput();
        }
    }

    /**
     * Met à jour une vidéo existante dans la bibliothèque.
     */
    public function update(Request $request, $locale, $id)
    {
        // Récupère la vidéo à mettre à jour ou renvoie une erreur 404 si elle n'existe pas
        $libraryVideo = LibraryVideo::findOrFail($id);

        // Validation des données envoyées dans la requête
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'department_id' => 'nullable|integer',
            'work_id' => 'nullable|integer',
            'date_creation' => 'nullable|date',
            'image' => 'nullable|image|max:20480',
            'video' => 'nullable|mimes:mp4,mov,ogg,qt|max:50000',
            'url' => 'nullable|string|max:512',
            'display_order' => 'required|integer',
        ]);

        // Ajoute le company_id à partir de la société courante
        $validatedData['company_id'] = $this->company->id;

        // Vérifie si un fichier image a été envoyé et le stocke
        if ($request->hasFile('image')) {
            // Supprime l'ancienne image si elle existe
            if ($libraryVideo->image) {
                \Log::info('Deleting old image: ' . $libraryVideo->image);
                Storage::delete('public/' . $libraryVideo->image);
            }

            // Stocke la nouvelle image et ajoute son chemin aux données validées
            $validatedData['image'] = $request->file('image')->store('images', 'public');
        }

        // Vérifie si un fichier vidéo a été envoyé et le stocke
        if ($request->hasFile('video')) {
            // Supprime l'ancienne vidéo si elle existe
            if ($libraryVideo->filename) {
                \Log::info('Deleting old video: ' . $libraryVideo->filename);
                Storage::delete('public/' . $libraryVideo->filename);
            }

            // Stocke la nouvelle vidéo et ajoute son chemin aux données validées
            $validatedData['filename'] = $request->file('video')->store('videos', 'public');
        }

        // Si aucun fichier vidéo ou image n'est envoyé, on conserve l'ancienne valeur
        $validatedData['filename'] = $validatedData['filename'] ?? $libraryVideo->filename;
        $validatedData['image'] = $validatedData['image'] ?? $libraryVideo->image;

        // Affiche les données validées pour le débogage
        // dd($validatedData);

        try {
            // Met à jour la vidéo avec les nouvelles données validées
            $libraryVideo->update($validatedData);

            // Redirige avec un message de succès
            return redirect()->route('training-library_video.index', ['locale' => $locale])
                ->with('success', __('messages.video_updated_successfully'));
        } catch (\Exception $e) {
            // En cas d'erreur, enregistre l'exception dans les logs et retourne avec un message d'erreur
            \Log::error('Erreur lors de la mise à jour de la vidéo : ' . $e->getMessage());
            return redirect()->back()->withErrors(['error_updating_video' => $e->getMessage()])->withInput();
        }
    }

    /**
     * Récupère la vidéo tutorielle pour une catégorie.
     */
    protected function get_tutorial_video($category, $locale)
    {
        $videoField = "video_$locale";
        return Storage::url($category->$videoField ?? 'videos/default-category-tutorial-video.mp4');
    }

    public function destroy($locale, $id)
    {
        try {
            // Trouver la vidéo à supprimer
            $libraryVideo = LibraryVideo::findOrFail($id);
    
            // Supprimer les historiques associés
            LibraryVideoHistory::where('library_video_id', $id)->delete();
    
            // Supprimer les fichiers liés
            if ($libraryVideo->image) {
                Storage::delete('public/' . $libraryVideo->image);
            }
            if ($libraryVideo->filename) {
                Storage::delete('public/' . $libraryVideo->filename);
            }
    
            // Supprimer la vidéo
            $libraryVideo->delete();
    
            return redirect()->route('training-library_video.list', ['locale' => $locale,'category_id'=>11])
                             ->with('success', 'Vidéo supprimée avec succès.');
        } catch (\Exception $e) {
            // Gérer les erreurs et rediriger avec un message d'erreur
            return redirect()->route('training-library_video.list', ['locale' => $locale ,'category_id'=>11])
                             ->with('error', 'Erreur lors de la suppression de la vidéo : ' . $e->getMessage());
        }
    }
    

    /**
     * Récupère les informations de complétion de la formation.
     */
    protected function completed_info($userTraining)
    {
        $completed = true;
        $lastStopTimestamp = null;

        foreach ($userTraining->track_records as $trackRecord) {
            $trackRecord->answer_points = $trackRecord->answer_points ?? -1;
            if ($trackRecord->answer_points == -1) {
                $completed = false;
                break;
            }
            if (!$lastStopTimestamp || $trackRecord->last_stop > $lastStopTimestamp) {
                $lastStopTimestamp = $trackRecord->last_stop;
            }
        }

        $date = $lastStopTimestamp ? date("Y-m-d", $lastStopTimestamp) : null;
        return ['completed' => $completed, 'date' => $date];
    }

    /**
     * Transforme une URL YouTube en URL d'intégration.
     */
    function getYouTubeEmbedUrl($url)
    {
        // Utilise une expression régulière pour extraire l'ID de la vidéo de l'URL
        preg_match('/(?:youtube\.com\/watch\?v=|youtu\.be\/|youtube\.com\/embed\/)([a-zA-Z0-9_-]{11})/', $url, $matches);

        if (!empty($matches[1])) {
            // Crée une URL d'intégration sans paramètres supplémentaires
            return 'https://www.youtube.com/embed/' . $matches[1];
        }

        return $url; // Si l'URL n'est pas reconnue, la renvoie telle quelle
    }
    public function saveHistory(Request $request)
{
    $validated = $request->validate([
        'user_id' => 'required|exists:users,id',
        'library_video_id' => 'required|exists:library_video,id',
        'start_time' => 'required|integer',
        'stop_time' => 'required|integer',
        'date' => 'required|date',
        'duration' => 'required|integer',
    ]);

    $historique = new LibraryVideoHistory();
    $historique->user_id = $validated['user_id'];
    $historique->library_video_id = $validated['library_video_id'];
    $historique->start_time = $validated['start_time'];
    $historique->stop_time = $validated['stop_time'];
    $historique->date = $validated['date'];
    $historique->duration = $validated['duration'];
    $historique->save();

    return response()->json(['message' => 'Historique sauvegardé avec succès']);
}

   

}
