<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use App\Models\DeviceIcons;
use App\Models\Devices;
use App\Models\Energies;
use App\Models\Instructions;
use App\Models\Mechanisms;
use App\Models\Positions;
use Illuminate\Http\Request;

class DevicesController extends Controller
{
    protected $icons;

    public function __construct(Request $request, DeviceIcons $icons)
    {
        parent::__construct($request);
        $this->icons = $icons;
    }

    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $devices = Devices::latest()
        ->with('icon')
        ->with('energy')
        ->with('mechanism')
        ->with('instruction')
        ->with('position')
        ->paginate(env('PAGINATION_DEFAULT_LIMIT', 10));

        return view('lockouts.devices.index', compact('devices'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $data = $this->getDependancies();

        return view('lockouts.devices.create', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validated = $request->validate([
            'device_id' => 'required|string|max:255|unique:devices,device_id',
            'icon_id' => 'required|exists:device_icons,id',
            'energy_id' => 'required|exists:energies,id',
            'position_id' => 'required|exists:positions,id',
            'description' => 'required|string|max:255',
            'emplacement' => 'required|string|max:255',
            'instruction_id' => 'required|exists:instructions,id',
            'mechanism_id' => 'nullable|exists:mechanisms,id',
            'position_decad_id' => 'required|string|max:255',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
        ]);

        if($request->has('need_lockpad')) {
            $validated['need_lockpad'] = true;
        } else {
            $validated['need_lockpad'] = false;
        }

        if($validated) {
            $imageName = time() . '_' . $request->file('image')->getClientOriginalName();
            $request->file('image')->storeAs(Devices::STORAGE, $imageName);

            $device = new Devices();
            $device->device_id = $validated['device_id'];
            $device->icon_id = $validated['icon_id'];
            $device->energy_id = $validated['energy_id'];
            $device->position_id = $validated['position_id'];
            $device->description = $validated['description'];
            $device->emplacement = $validated['emplacement'];
            $device->instruction_id = $validated['instruction_id'];
            $device->mechanism_id = $validated['mechanism_id'];
            $device->position_decad_id = $validated['position_decad_id'];
            $device->need_lockpad = $validated['need_lockpad'];
            $device->image = $imageName;
            $device->company_id = auth()->user()->company_id;
            $device->save();
        }

        return redirect()->route('devices.index', app()->getLocale())->with('success', __('devices.message.success.create'));
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Devices $device)
    {
        return view('lockouts.devices.show', compact('device'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, Devices $device)
    {
        $data = $this->getDependancies();

        return view('lockouts.devices.edit', $data + compact('device'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Devices $device)
    {
        $validated = $request->validate([
            'device_id' => 'required|string|max:255|unique:devices,device_id,' . $device->id,
            'icon_id' => 'required|exists:device_icons,id',
            'energy_id' => 'required|exists:energies,id',
            'position_id' => 'required|exists:positions,id',
            'description' => 'required|string|max:255',
            'emplacement' => 'required|string|max:255',
            'instruction_id' => 'required|exists:instructions,id',
            'mechanism_id' => 'nullable|exists:mechanisms,id',
            'position_decad_id' => 'required|string|max:255',
            'image' => 'nullable|mimes:jpeg,png,jpg,gif,svg|max:2048',
        ]);

        if($request->has('need_lockpad')) {
            $validated['need_lockpad'] = true;
        } else {
            $validated['need_lockpad'] = false;
        }

        if($validated) {
            $device->update($validated);
        }

        return redirect()->route('devices.index', app()->getLocale())->with('success', __('devices.message.success.update'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Devices $device)
    {
        $device->delete();

        return redirect()->route('devices.index', app()->getLocale())->with('success', __('devices.message.success.delete'));
    }

    public function getDependancies()
    {
        $icons = DeviceIcons::forCompany(['id', 'description_fr', 'description_en']);

        $energies = Energies::forCompany(['id', 'label_fr', 'label_en']);

        $positions = Positions::forCompany(['id', 'label_fr', 'label_en']);

        $instructions = Instructions::forCompany(['id', 'label_fr', 'label_en']);

        $mechanisms = Mechanisms::forCompany(['id', 'type']);

        return compact('icons', 'energies', 'positions', 'instructions', 'mechanisms');
    }
}
