<?php

namespace App\Http\Controllers\Lockouts;

use App\Models\Mechanisms;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Storage;

class MechanismsController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        return view('lockouts.mechanisms.index', [
            'mechanisms' => Mechanisms::latest('id')->paginate(env('PAGINATION_DEFAULT_LIMIT')),
        ]);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('lockouts.mechanisms.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validated = $request->validate([
            'type' => 'bail|required|string|max:255',
            "icon" => 'bail|required|image|max:255'
        ]);

        $iconName = time() . '_' . $request->file('icon')->getClientOriginalName();
        $request->file('icon')->storeAs(Mechanisms::STORAGE, $iconName);

        if($validated) {
            $mechanism = Mechanisms::create([
                'type' => $validated['type'],
                'icon' => $iconName,
                'company_id' => auth()->user()->company->id
            ]);

            $mechanism->save();
        }

        return redirect(route("mechanisms.index", app()->getLocale()));
    }

    /**
     * Display the specified resource.
     * @param Mechanisms $mechanisms
     * @return \Illuminate\Contracts\View\View
     */
    public function show($locale, Mechanisms $mechanisms)
    {
        return view('lockouts.mechanisms.show', compact('mechanisms'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param Mechanisms $mechanisms
     * @return \Illuminate\Contracts\View\View
     */
    public function edit($locale, Mechanisms $mechanisms)
    {
        // Vérifier si l'utilisateur a l'autorisation de modifier ce mécanisme
        $this->isAuthorized($mechanisms);

        // Charger la vue d'édition avec les données du mécanisme
        return view('lockouts.mechanisms.edit', compact('mechanisms'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Mechanisms $mechanisms)
    {
        // Vérifier si l'utilisateur a l'autorisation de modifier ce mécanisme
        $this->isAuthorized($mechanisms);

        // Valider les données de la requête
        $validated = $request->validate([
            'type' => 'required|string|max:255',
            'icon' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048'
        ]);

        // Mettre à jour le type
        $mechanisms->type = $validated['type'];

        // Gérer l'icône si une nouvelle est fournie
        if ($request->hasFile('icon')) {
            $iconFile = $request->file('icon');

            // Vérifier si le fichier est une image valide
            if ($iconFile->isValid()) {
                // Supprimer l'ancienne icône si elle existe
                if ($mechanisms->icon && $mechanisms->icon !== Mechanisms::DEFAULT_ICON) {
                    Storage::delete(Mechanisms::STORAGE . '/' . $mechanisms->icon);
                }

                // Stocker la nouvelle icône
                $iconName = time() . '_' . $iconFile->getClientOriginalName();
                $iconFile->storeAs(Mechanisms::STORAGE, $iconName);
                $mechanisms->icon = $iconName;
            } else {
                return back()->withErrors(['icon' => 'Le fichier d\'icône n\'est pas valide.'])->withInput();
            }
        }

        // Sauvegarder les modifications
        $mechanisms->save();

        // Rediriger vers la page d'index avec un message de succès
        return redirect()->route('mechanisms.index', ['locale' => app()->getLocale()])
                         ->with('success', 'Mécanisme mis à jour avec succès.');
    }

    /**
     * Remove the specified resource from storage.
     * @param Mechanisms $mechanisms
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy($locale, Mechanisms $mechanisms)
    {
        // Vérifier si l'utilisateur a l'autorisation de supprimer ce mécanisme
        $this->isAuthorized($mechanisms);

        // Supprimer l'icône associée si elle existe et n'est pas l'icône par défaut
        if ($mechanisms->icon && $mechanisms->icon !== Mechanisms::DEFAULT_ICON) {
            Storage::delete(Mechanisms::STORAGE . '/' . $mechanisms->icon);
        }

        // Supprimer le mécanisme de la base de données
        $mechanisms->delete();

        // Rediriger vers la page d'index avec un message de succès
        return redirect()->route('mechanisms.index', ['locale' => app()->getLocale()])
                         ->with('success', 'Mécanisme supprimé avec succès.');
    }

    /**
     * @param Mechanisms $mechanisms
     *
     * @return void
     */
    private function isAuthorized(Mechanisms $mechanisms): void
    {
        if (auth()->user()->company->id !== $mechanisms->company_id) {
            abort(403, 'Accès non autorisé.');
        }
    }
}
